﻿using FluentValidation.AspNetCore;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Options;
using Newtonsoft.Json.Linq;
using Performance.DtoModels;
using Performance.DtoModels.AppSettings;
using Performance.Infrastructure;
using Performance.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Performance.Api.Controllers
{
    [Route("api/[controller]")]
    public class AccountController : Controller
    {
        UserService _userService;
        IMemoryCache _memoryCache;
        Application _options;
        public AccountController(UserService userService,
            IMemoryCache memoryCache,
            IOptions<Application> options)
        {
            _userService = userService;
            _memoryCache = memoryCache;
            _options = options.Value;
        }

        /// <summary>
        /// 登录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [NoVerify]
        [HttpPost]
        [Route("login")]
        public ApiResponse<UserIdentity> Login([FromBody]LoginRequest request)
        {
            var user = _userService.Login(request);

            if (string.IsNullOrEmpty(user.Token))
                user.Token = Guid.NewGuid().ToString("N");

            var option = new MemoryCacheEntryOptions()
            {
                SlidingExpiration = TimeSpan.FromMinutes(_options.ExpirationMinutes)
            };
            _memoryCache.Set(user.Token, user, option);

            return new ApiResponse<UserIdentity>(ResponseType.OK, user);
        }

        /// <summary>
        /// 新增用户
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("insert")]
        [HttpPost]
        public ApiResponse Insert(
            [CustomizeValidator(RuleSet = "Insert")]
            [FromBody]UserRequest request)
        {
            if (!_userService.Insert(request))
                return new ApiResponse(ResponseType.Fail);
            return new ApiResponse(ResponseType.OK);
        }

        /// <summary>
        /// 修改用户
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("update")]
        [HttpPost]
        public ApiResponse Update(
            [CustomizeValidator(RuleSet = "Update")]
            [FromBody]UserRequest request)
        {
            if (!_userService.Update(request))
                return new ApiResponse(ResponseType.Fail);
            return new ApiResponse(ResponseType.OK);
        }

        /// <summary>
        /// 设置用户管辖医院
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("sethospital")]
        [HttpPost]
        public ApiResponse SetHospital([FromBody]SetHospitalRequest request)
        {
            if (!_userService.SetHospital(request))
                return new ApiResponse(ResponseType.Fail);
            return new ApiResponse(ResponseType.OK);
        }
    }
}
